// Copyright (c) <2012> <Leif Asbrink>
//
// Permission is hereby granted, free of charge, to any person 
// obtaining a copy of this software and associated documentation 
// files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of 
// the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be 
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
// OR OTHER DEALINGS IN THE SOFTWARE.


#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "globdef.h"

#if(OSNUM == OSNUM_WINDOWS)
#include <windows.h>
#include <winsock.h>
#define RECV_FLAG 0
#ifndef IP_ADD_MEMBERSHIP
#define IP_ADD_MEMBERSHIP 12
#endif
#define INVSOCK INVALID_SOCKET
#define CLOSE_FD closesocket
typedef struct{
struct in_addr imr_multiaddr;   /* IP multicast address of group */
struct in_addr imr_interface;   /* local IP address of interface */
}IP_MREQ;
extern WSADATA wsadata;

#endif

#if(OSNUM == OSNUM_LINUX)
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <string.h>
#include <stdio.h>
#include <netdb.h>
#include <unistd.h>
#include <semaphore.h>
#include <pthread.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#ifdef MSG_WAITALL
#define RECV_FLAG MSG_WAITALL
#else
#define RECV_FLAG 0
#endif
#define IP_MREQ struct ip_mreq
#define INVSOCK -1
#define CLOSE_FD close
#endif

#include "uidef.h"
#include "sdrdef.h"
#include "vernr.h"
#include "rusage.h"
#include "thrdef.h"
#include "fft1def.h"
#include "screendef.h"

void net_write(FD fd, void *buf, int size);

P_SDRIP sdrip;

char *sdrip_parfil_name="par_sdrip";
char *sdrip_parm_text[MAX_SDRIP_PARM]={"IP1",
                                       "IP2",
                                       "IP3",
                                       "IP4",
                                       "Port",
                                       "Rate dividend",
                                       "Format",
                                       "UDP size",
                                       "Dither",
                                       "ADgain",
                                       "RF filter",
                                       "Connect",
                                       "Clock adjust",
                                       "Check"
                                       }; 
char *pkgsize[2]={"Large","Small"};
char *onoff[2]={"OFF","ON"};
char *adgains[2]={"1.0","1.5"};
char *inp_connect[2]={"Normal","Converter"};
char *rffil[2]={"Auto","Bypass"};

FD sdrip_fd;
struct sockaddr_in sdrip_addr;

int verify_sdrip_parameters(void)
{
float t1;
if(sdrip.ip0 < 0 || sdrip.ip0 > 255 ||
   sdrip.ip1 < 0 || sdrip.ip1 > 255 ||
   sdrip.ip2 < 0 || sdrip.ip2 > 255 ||
   sdrip.ip3 < 0 || sdrip.ip3 > 255 ||
   sdrip.port < 0 || sdrip.port > 65535 ||
   sdrip.udp_size < 0 || sdrip.udp_size > 1 ||
   sdrip.dither < 0 || sdrip.dither > 1 ||
   sdrip.adgain < 0 || sdrip.adgain > 1 ||
   sdrip.rf_filter < 0 || sdrip.rf_filter > 1 ||
   sdrip.connect < 0 || sdrip.connect > 1 ||
   abs(sdrip.clock_adjust) > 10000 ||
   sdrip.check != SDRIP_PAR_VERNR)
  {
  return FALSE;
  }
else
  {
  if(sdrip.format != 16 && sdrip.format != 24)return FALSE;
  t1=SDRIP_SAMPLING_CLOCK_DIV10/sdrip.rate_dividend;
  if(t1 < 32000)return FALSE;
  if(sdrip.format == 16)
    {
    if(t1 > 2000001.)return FALSE;
    }
  else
    {
    if(t1 > 1333334.0)return FALSE;
    }
  return TRUE;
  }
}


int display_sdrip_parm_info(int *line)
{
char s[80];
int errcod;
errcod=read_sdrpar(sdrip_parfil_name, MAX_SDRIP_PARM, 
                                     sdrip_parm_text, (int*)((void*)&sdrip));
if(errcod == 0)
  {
  settextcolor(7);
  sprintf(s,"IP address         = %d.%d.%d.%d", 
                          sdrip.ip0,sdrip.ip1,sdrip.ip2,sdrip.ip3);
  lir_text(24,line[0],s);
  line[0]++;
  SNDLOG"\n%s",s);
  sprintf(s,"Port               = %d",sdrip.port);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"Sampling speed     = %.0f kHz   (dividend=%d)",
                       8000.0/sdrip.rate_dividend,sdrip.rate_dividend);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"Bits per sample    = %d",sdrip.format);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"UDP packet size    = %s",pkgsize[sdrip.udp_size]);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"Dither             = %s",onoff[sdrip.dither]);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"A/D gain           = %s",adgains[sdrip.adgain]);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"RF filter          = %s",rffil[sdrip.rf_filter]);
  lir_text(24,line[0],s);
  line[0]++;
  sprintf(s,"Input connection   = %s",inp_connect[sdrip.connect]);
  lir_text(24,line[0],s);
  }
return (errcod);
}


void open_sdrip(void)
{
int i, j;
unsigned char *ip;
char s[80];
sdr=-1;
#if(OSNUM == OSNUM_WINDOWS)
if(WSAStartup(2, &wsadata) != 0)
  {
  lirerr(1263);
  return;   
  }
#endif
memset(&sdrip_addr, 0, sizeof(sdrip_addr)); 
sdrip_addr.sin_family=AF_INET;
sdrip_addr.sin_port=htons(sdrip.port);
ip=(unsigned char*)&sdrip_addr.sin_addr.s_addr;
ip[0]=sdrip.ip0;
ip[1]=sdrip.ip1;
ip[2]=sdrip.ip2;
ip[3]=sdrip.ip3;
if ((sdrip_fd = socket(AF_INET, SOCK_STREAM, 0)) == INVSOCK) 
  {
  lirerr(1249);
  return;
  }
j=connect(sdrip_fd, (struct sockaddr *)&sdrip_addr, sizeof(sdrip_addr));
lir_sleep(30000);
i=0;
lir_inkey=0;
while(j == -1 && lir_inkey == 0)
  {
  if(i > 4)sprintf(s,"Trying to connect %d,  errno=%d",i-4,errno);
  lir_text(1,5,s);
  lir_refresh_screen();
  j=connect(sdrip_fd, (struct sockaddr *)&sdrip_addr, sizeof(sdrip_addr));
  test_keyboard();
  if(kill_all_flag)return;
  i++;
  lir_sleep(30000);
  }
if(lir_inkey != 0)
  {
  lir_inkey=0;
  return;
  }
sdr=0;
}

void sdrip_setup_header(int *line)
{
char s[80];
clear_screen();
line[0]=2;
settextcolor(13);
lir_text(20,line[0],"Installation routine for SDR-IP");
settextcolor(7);
line[0]+=2;
sprintf(s,"IP address = %d.%d.%d.%d",sdrip.ip0,sdrip.ip1,sdrip.ip2,sdrip.ip3);
lir_text(10,line[0],s);
line[0]++;
sprintf(s,"Port = %d",sdrip.port);
lir_text(16,line[0],s);
line[0]+=2;
lir_refresh_screen();
}

void close_sdrip(void)
{
CLOSE_FD(sdrip_fd);
sdr=-1;
}

void init_sdrip(void)
{
int i, errcod, line, length;
char msg[20], s[80], textbuf[80];
unsigned b0, b1, b2, b3;
unsigned char c;
float t1;
FILE *sdrip_file;
int *sdr_pi;
errcod=read_sdrpar(sdrip_parfil_name, MAX_SDRIP_PARM, 
                                     sdrip_parm_text, (int*)((void*)&sdrip));
if(errcod != 0)
  {
  sdrip.ip0=10;
  sdrip.ip1=0;
  sdrip.ip2=0;
  sdrip.ip3=93;
  sdrip.port=50000;
  sdrip.rate_dividend=100;
  sdrip.format=24;
  sdrip.udp_size=0;
  sdrip.dither=0;
  sdrip.adgain=0;
  sdrip.connect=0;
  sdrip.clock_adjust=0;  
  sdrip.check=SDRIP_PAR_VERNR;
  }
restart:;  
sdrip_setup_header(&line);
lir_text(0,line,"Press MENU button on the SDR-IP and set 'TCP/UDP Settings' to");
line++;
lir_text(0,line,"'Manual Both Same' with address and port as specified above");
line++;
lir_text(0,line,"or change address and port to agree with your SDR-IP");
line+=4;
lir_text(12,line,"A=Change IP address.");
line++;
lir_text(12,line,"B=Change port.");
line++;
lir_text(12,line,"C=Connect to the SDR-IP");
line++;
lir_text(12,line,"X=Skip and return to previous menu.");
line+=2;
await_processed_keyboard();
if(kill_all_flag)return;
switch (lir_inkey)
  {
  case 'A':
specify:
  lir_text(10,line,"Specify new IP address:");
  length=lir_get_text(40, line, textbuf);
  if(kill_all_flag) return;
// validate format of IP address 
  if ((length < 7) || (length > 15))  goto format_error;
  if (strspn(textbuf, "0123456789.") < strlen(textbuf)) goto format_error;
  if (sscanf(textbuf, "%3u.%3u.%3u.%3u%c", &b0, &b1, &b2, &b3, &c) != 4)
                                                        goto format_error;
  if ((b0 | b1 | b2 | b3) > 255)
    {
format_error:;
    lir_text(10,line+1,"IP address format-error: Try again ");
    goto specify; 
    }
  sdrip.ip0=b0;
  sdrip.ip1=b1;
  sdrip.ip2=b2;
  sdrip.ip3=b3;  
  break;
  
  case 'B':
  lir_text(10,line,"New port number:");
  sdrip.port=lir_get_integer(28,line,5,0,65535);
  break;
  
  case 'C':
  clear_screen();
  open_sdrip();
  if(sdr == 0)goto verify_open;
  break;
  
  case 'X':
  return;
  }
if(kill_all_flag)return;  
goto restart;
verify_open:;
sdrip_setup_header(&line);
msg[0]=0x04;
msg[1]=0x20;
msg[2]=0x01;
msg[3]=0x00;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
sprintf(s,"Target Name = %s",&textbuf[4]);
lir_text(9,line,s);  
line++;
lir_refresh_screen();
msg[2]=0x02;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
sprintf(s,"Serial number = %s",&textbuf[4]);
lir_text(7,line,s);  
line++;
lir_refresh_screen();
msg[2]=0x03;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
t1=256*textbuf[5]+textbuf[4];
sprintf(s,"Interface version = %f",0.01*t1);
lir_text(3,line,s);  
line++;
lir_refresh_screen();
msg[2]=0x04;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
t1=256*textbuf[5]+textbuf[4];
sprintf(s,"Boot code version = %f",0.01*t1);
lir_text(3,line,s);  
line++;
lir_refresh_screen();
msg[3]=0x01;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
t1=256*textbuf[5]+textbuf[4];
sprintf(s,"Firmware version = %f",0.01*t1);
lir_text(4,line,s);  
line++;
lir_refresh_screen();
msg[3]=0x02;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
t1=256*textbuf[5]+textbuf[4];
sprintf(s,"Hardware version = %f",0.01*t1);
lir_text(4,line,s);  
line++;
lir_refresh_screen();
msg[2]=0x09;
msg[3]=0x00;
net_write(sdrip_fd,msg,4);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
t1=256*textbuf[5]+textbuf[4];
sprintf(s,"Product ID [%x][%x][%x][%x]",
                  textbuf[4],textbuf[5],textbuf[6],textbuf[7]);
lir_text(4,line,s);  
line+=2;
lir_text(10,line,"Use this hardware? (Y/N)");
lir_inkey=0;
while(lir_inkey != 'Y')
  {
  if(kill_all_flag)goto sdrip_errexit;
  await_processed_keyboard();
  if(lir_inkey == 'N')goto restart;
  }
setpar:;  
sdrip_setup_header(&line);
lir_text(0,line,"Set global parameters for the SDR-IP");
line++;
sprintf(s,"A => Rate dividend = %d (for a sampling rate of %.0f kHz)",
                             sdrip.rate_dividend, 8000.0/sdrip.rate_dividend);
lir_text(0,line,s);
line++;
sprintf(s,"B => Bits per sample = %d",sdrip.format);
lir_text(0,line,s);
line++;
sprintf(s,"C => UDP packet size = %s",pkgsize[sdrip.udp_size]);
lir_text(0,line,s);
line++;
sprintf(s,"D => Dither %s",onoff[sdrip.dither]);
lir_text(0,line,s);
line++;
sprintf(s,"E => A/D gain %s",adgains[sdrip.adgain]);
lir_text(0,line,s);
line++;
sprintf(s,"F => Input connection %s",inp_connect[sdrip.connect]);
lir_text(0,line,s);
line++;
sprintf(s,"X => Save settings in %s and exit",sdrip_parfil_name);
lir_text(0,line,s);
line++;
await_processed_keyboard();
if(kill_all_flag)goto sdrip_errexit;
switch (lir_inkey)
  {
  case 'A':
  if(sdrip.format == 16)
    {
    i=4;
    }
  else
    {
    i=6;
    }  
  sprintf(s,"Enter new rate dividend (%d to 250) =>",i);
  lir_text(0,line+1,s);
  sdrip.rate_dividend=lir_get_integer(37,line+1,3,i,250);
  clear_lines(line+1,line+1);
  break;
  
  case 'B':
  if(sdrip.format == 16)
    {
    sdrip.format=24;
    }
  else
    {
    sdrip.format=16;
    }  
  break;
  
  case 'C':
  sdrip.udp_size^=1;
  break;
  
  case 'D':
  sdrip.dither^=1;
  break;
  
  case 'E':
  sdrip.adgain^=1;
  break;
  
  case 'F':
  sdrip.connect^=1;
  break;
  
  case 'X':
  adjusted_sdr_clock=10.*SDRIP_SAMPLING_CLOCK_DIV10+sdrip.clock_adjust;
  ui.rx_ad_speed=adjusted_sdr_clock/(10*sdrip.rate_dividend);
  ui.rx_input_mode=IQ_DATA+DIGITAL_IQ;
  if(sdrip.format == 24)ui.rx_input_mode+=DWORD_INPUT;
  ui.rx_rf_channels=1;
  ui.rx_ad_channels=2;
  ui.rx_admode=0;
  sdrip.check=SDRIP_PAR_VERNR;
  sdrip_file=fopen(sdrip_parfil_name,"w");
  if(sdrip_file == NULL)
    {
sdrip_errexit:;
    CLOSE_FD(sdrip_fd);
#if(OSNUM == OSNUM_WINDOWS)
    WSACleanup();
#endif    
    clear_screen();
    return;
    }
  sdr_pi=(int*)(&sdrip);
  for(i=0; i<MAX_SDRIP_PARM; i++)
    {
    fprintf(sdrip_file,"%s [%d]\n",sdrip_parm_text[i],sdr_pi[i]);
    }
  parfile_end(sdrip_file);
  ui.rx_addev_no=SDRIP_DEVICE_CODE;
#if(OSNUM == OSNUM_WINDOWS)
  WSACleanup();
#endif    
  return;
  }
goto setpar;   
}

void set_sdrip_att(void)
{
char msg[20], textbuf[80];
msg[0]=0x06;
msg[1]=0x00;
msg[2]=0x38;
msg[3]=0x00;
msg[4]=0x00;
msg[5]=fg.gain;
net_write(sdrip_fd,msg,6);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
}

void set_sdrip_frequency(void)
{
double dt1;
char msg[20], textbuf[80];
unsigned int *sr;
unsigned int ii;
msg[0]=0x0a;
msg[1]=0x00;
msg[2]=0x20;
msg[3]=0x00;
msg[4]=0x00;
sr=(unsigned int*)&msg[6];  
dt1=fg.passband_center*1000000+0.5;
if(dt1 < 0 || dt1 > adjusted_sdr_clock/2)
  {
  fg.passband_center=7.05;
  dt1=fg.passband_center*1000000;
  }
sr[0]=dt1/256;
dt1-=256*sr[0];
ii=dt1;
if(ii > 255)ii=255;
msg[5]=ii;
net_write(sdrip_fd,msg,10);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
msg[4]=0x01;
net_write(sdrip_fd,msg,10);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
}

void sdrip_input(void)
{
#if RUSAGE_OLD == TRUE
int local_workload_counter;
#endif
struct sockaddr_in udp_addr;
FD udp_fd;
int i,errcod;
int rxin_local_workload_reset;
int pkg_size;
int timf1_netptr;
short int pkg_chk;
int sdrip_mode;
int *sr;
char *tmpbuf;
char s[80], msg[20], textbuf[80];
double dt1, read_start_time, total_reads;
short int *ichk;
int timing_loop_counter,timing_loop_counter_max,initial_skip_flag;
float t1;
int local_att_counter;
int local_nco_counter;
short int pkg_number;
char optval;
#if OSNUM == OSNUM_LINUX
clear_thread_times(THREAD_SDRIP_INPUT);
#endif
#if RUSAGE_OLD == TRUE
local_workload_counter=workload_counter;
#endif
local_att_counter=sdr_att_counter;
local_nco_counter=sdr_nco_counter;
errcod=0;
pkg_number=0;
dt1=current_time();
errcod=read_sdrpar(sdrip_parfil_name, MAX_SDRIP_PARM, 
                                     sdrip_parm_text, (int*)((void*)&sdrip));
if(errcod != 0)
  {
parerr:;  
  errcod=1170;
  goto sdrip_init_error_exit;
  }
i=verify_sdrip_parameters();
if(i == FALSE)goto parerr;
while(sdr == -1)
  {
  open_sdrip();
  lir_sleep(30000);
  if(sdr == -1)
    {
    sprintf(s,"Waiting %.2f", current_time()-dt1);
    lir_text(0,screen_last_line,s);
    lir_refresh_screen();
    if(kill_all_flag)goto sdrip_init_error_exit;
    }
  }
adjusted_sdr_clock=10.*SDRIP_SAMPLING_CLOCK_DIV10+sdrip.clock_adjust;
t1=ui.rx_ad_speed-adjusted_sdr_clock/(10*sdrip.rate_dividend);
if(fabs(t1/ui.rx_ad_speed) > 0.0001)
  {
  errcod=1330;
  goto sdrip_error_exit;
  }
// Set the DDC output sampling rate.
msg[0]=0x09;
msg[1]=0x00;
msg[2]=0xb8;
msg[3]=0x00;
msg[4]=0x00;
sr=(int*)&msg[5];  
sr[0]=SDRIP_SAMPLING_CLOCK_DIV10/sdrip.rate_dividend;
net_write(sdrip_fd,msg,9);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
// Set the RF filter mode.
msg[0]=0x06;
msg[1]=0x00;
msg[2]=0x44;
msg[3]=0x00;
msg[4]=0x00;
if(sdrip.rf_filter == 0)
  {
  msg[5]=0x00;
  }
else
  {
  msg[5]=0x0b;
  }    
net_write(sdrip_fd,msg,6);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
// Set the A/D mode.
msg[0]=0x06;
msg[1]=0x00;
msg[2]=0x8a;
msg[3]=0x00;
msg[4]=0x00;
if(sdrip.dither != 0)msg[4]|=1;
if(sdrip.adgain != 0)msg[4]|=2;
if(sdrip.connect != 0)msg[4]|=4;
msg[5]=0x01;
net_write(sdrip_fd,msg,6);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
// Set the data output packet size.
msg[0]=0x05;
msg[1]=0x00;
msg[2]=0xc4;
msg[3]=0x00;
msg[4]=sdrip.udp_size;
sdrip_mode=sdrip.udp_size;
net_write(sdrip_fd,msg,5);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
set_sdrip_att();
set_sdrip_frequency();
fft1_block_timing();
fft1_hz_per_point=(float)ui.rx_ad_speed/fft1_size;
if(thread_command_flag[THREAD_SCREEN]!=THRFLAG_NOT_ACTIVE)
  {
  while(thread_status_flag[THREAD_SCREEN]!=THRFLAG_ACTIVE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_IDLE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_SEM_WAIT)
    {
    if(thread_command_flag[THREAD_SDRIP_INPUT] ==
                                           THRFLAG_KILL)goto sdrip_error_exit;
    lir_sleep(10000);
    }
  }
// Set the receiver state.
msg[0]=0x08;
msg[1]=0x00;
msg[2]=0x18;
msg[3]=0x00;
msg[4]=0x80;        // Set complex contigous data.
msg[5]=0x02;        // set run/stop to RUN
if(sdrip.format == 16)
  {
  msg[6]=0x00;
  }
else
  {
  sdrip_mode|=2;
  msg[6]=0x80;  
  }
msg[7]=1;
net_write(sdrip_fd,msg,8);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
if ((udp_fd=socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP)) == INVSOCK) 
  {
  lirerr(1284);
  goto sdrip_exit;
  }
optval=TRUE;
setsockopt( udp_fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval) );
optval=TRUE;
//setsockopt( udp_fd, SOL_SOCKET, SO_DONTLINGER, &optval, sizeof(optval) );
// Set a big RX buffer. Defaults: Linux=109568 and Windows=8192.
// Set the TX buffer small. We will not send on this socket.
i=131072;
setsockopt(udp_fd,SOL_SOCKET,SO_RCVBUF,(char*)&i,sizeof(int));
i=8192;
setsockopt(udp_fd,SOL_SOCKET,SO_SNDBUF,(char*)&i,sizeof(int));
memset(&udp_addr, 0, sizeof(udp_addr)); 
udp_addr.sin_family=AF_INET;
udp_addr.sin_port=htons(sdrip.port);
udp_addr.sin_addr.s_addr = htonl(INADDR_ANY );
// bind to the SDR-IP address and port.
if (bind(udp_fd, (struct sockaddr *)&udp_addr, sizeof(udp_addr)) < 0) 
  {
  lirerr(1302);
  goto sdrip_udp_exit;
  }
tmpbuf=malloc(1444);
if(tmpbuf == NULL)goto sdrip_udp_exit;
// sdrip_mode == 0 => large UDP, 16 bit
// sdrip_mode == 1 => small UDP, 16 bit
// sdrip_mode == 2 => large UDP, 24 bit
// sdrip_mode == 3 => small UDP, 24 bit
timf1_netptr=timf1p_pa;
switch (sdrip_mode)
  {
  case 0:
  pkg_size=1028;
  pkg_chk=0x8404;
  break;

  case 1:
  pkg_size=516;
  pkg_chk=0x8204;

  break;

  case 2:
  pkg_size=1444;
  pkg_chk=0x85a4;

  break;

  case 3:
  pkg_size=388;
  pkg_chk=0x8184;

  break;
  
  default:
  pkg_size=0;
  pkg_chk=0;
  lirerr(46295);
  goto sdrip_udpbuf_exit;
  }
#include "timing_setup.c"
thread_status_flag[THREAD_SDRIP_INPUT]=THRFLAG_ACTIVE;
while(thread_command_flag[THREAD_SDRIP_INPUT] == THRFLAG_ACTIVE)
  {
#if RUSAGE_OLD == TRUE
  if(local_workload_counter != workload_counter)
    {
    local_workload_counter=workload_counter;
    make_thread_times(THREAD_SDRIP_INPUT);
    }
#endif
  if(local_att_counter != sdr_att_counter)
    {
    if(thread_status_flag[THREAD_SYSCALL] == THRFLAG_SEM_WAIT &&
       thread_command_flag[THREAD_SYSCALL] == THRFLAG_ACTIVE)
      {
      thread_command_flag[THREAD_SYSCALL]=THRFLAG_SET_SDRIP_ATT;
      lir_set_event(EVENT_SYSCALL);
      }
    if( thread_status_flag[THREAD_SYSCALL] == THRFLAG_IDLE &&
        thread_command_flag[THREAD_SYSCALL]==THRFLAG_SET_SDRIP_ATT)
      {
      local_att_counter=sdr_att_counter;
      thread_command_flag[THREAD_SYSCALL]=THRFLAG_ACTIVE;
      lir_set_event(EVENT_SYSCALL);
      }
    }
  if(local_nco_counter != sdr_nco_counter)
    {
    if(thread_status_flag[THREAD_SYSCALL] == THRFLAG_SEM_WAIT &&
       thread_command_flag[THREAD_SYSCALL] == THRFLAG_ACTIVE)
      {
      thread_command_flag[THREAD_SYSCALL]=THRFLAG_SET_SDRIP_FREQUENCY;
      lir_set_event(EVENT_SYSCALL);
      }
    if( thread_status_flag[THREAD_SYSCALL] == THRFLAG_IDLE &&
        thread_command_flag[THREAD_SYSCALL]==THRFLAG_SET_SDRIP_FREQUENCY)
      {
      local_nco_counter=sdr_nco_counter;
      thread_command_flag[THREAD_SYSCALL]=THRFLAG_ACTIVE;
      lir_set_event(EVENT_SYSCALL);
      }
    }
  i=recv(udp_fd, tmpbuf, pkg_size, 0);
  ichk=(short int*)tmpbuf;
  if(ichk[0] != pkg_chk)
    {
    lirerr(8429256);
    goto sdrip_udpbuf_exit;
    }
// In case the packet number is totally wrong, just
// set pkg_number to the current package number.
  if(pkg_number-ichk[1] < 0 || pkg_number-ichk[1]> 15)pkg_number=ichk[1];    
// Fill zeroes in case the packet number does not agree
// with pkg_number and increase pkg_number until we
// reach agreement. Then fill the current data from the buffer.
  if(sdrip.format == 16)
    {
    while(pkg_number != ichk[1])
      {
      for(i=4; i<pkg_size; i++)
        {
        timf1_char[timf1_netptr]=0;
        timf1_netptr=(timf1_netptr+1)&timf1_bytemask;
        }
      pkg_number++;
      }
    for(i=4; i<pkg_size; i++)
      {
      timf1_char[timf1_netptr]=tmpbuf[i];
      timf1_netptr=(timf1_netptr+1)&timf1_bytemask;
      }
    pkg_number++;
    
    }
  else
    {
    while(pkg_number != ichk[1])
      {
      for(i=4; i<pkg_size; i+=3)
        {
        timf1_char[timf1_netptr  ]=0;
        timf1_char[timf1_netptr+1]=0;
        timf1_char[timf1_netptr+2]=0;
        timf1_char[timf1_netptr+3]=0;
        timf1_netptr=(timf1_netptr+4)&timf1_bytemask;
        }
      pkg_number++;
      }
    for(i=4; i<pkg_size; i+=3)
      {
      timf1_char[timf1_netptr  ]=0;
      timf1_char[timf1_netptr+1]=tmpbuf[i  ];
      timf1_char[timf1_netptr+2]=tmpbuf[i+1];
      timf1_char[timf1_netptr+3]=tmpbuf[i+2];
      timf1_netptr=(timf1_netptr+4)&timf1_bytemask;
      }
    pkg_number++;
    }  
  while( ((timf1_netptr-timf1p_pa+timf1_bytes)&timf1_bytemask) > 
                                     2*snd[RXAD].block_bytes)
    {
#include "input_speed.c"
    finish_rx_read();
    }    
  if(kill_all_flag) goto sdrip_udpbuf_exit;
  }
sdrip_udpbuf_exit:;
free(tmpbuf);  
sdrip_udp_exit:;
CLOSE_FD(udp_fd);
sdrip_exit:;
// Set the receiver state.
msg[0]=0x08;
msg[1]=0x00;
msg[2]=0x18;
msg[3]=0x00;
msg[4]=0x80;        // Set complex contigous data.
msg[5]=0x01;        // set run/stop to STOP
if(sdrip.format == 16)
  {
  msg[6]=0x00;
  }
else
  {
  msg[6]=0x80;  
  }
msg[7]=0x01;  
net_write(sdrip_fd,msg,8);
lir_sleep(100000);
recv(sdrip_fd,textbuf,80,0);
sdrip_error_exit:;
close_sdrip();
sdrip_init_error_exit:;
if(errcod != 0)lirerr(errcod);
thread_status_flag[THREAD_SDRIP_INPUT]=THRFLAG_RETURNED;
while(thread_command_flag[THREAD_SDRIP_INPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
}
